# lab3_Graphs.R
# command file for analyzing individual hue naming data
# Lew Harvey
# 16 February 2010

# read file into data frame
df <- read.delim(file.choose(), header=TRUE)

# Compute opponent processs and add columns to data frame
df <- cbind(df, red_green = df$red - df$green)
df <- cbind(df, yellow_blue = df$yellow - df$blue)
df <- cbind(df, total = df$red + df$yellow + df$green + df$blue)

# Compute the mean wavelengths of the pure colors
pure.blue   <- mean(df$pure_blue,   na.rm=TRUE)
pure.green  <- mean(df$pure_green,  na.rm=TRUE)
pure.yellow <- mean(df$pure_yellow, na.rm=TRUE)

# fit a polynomial to the opponent process data and
rg.poly <- lm(red_green ~ poly(wavelength, degree = 8), data = df)
yb.poly <- lm(yellow_blue ~ poly(wavelength, degree = 8), data = df)

# define the functions for finding the roots
# of the opponent process equations
predict.rg <- function(nm){predict(rg.poly, data.frame(wavelength = nm))}
predict.yb <- function(nm){predict(yb.poly, data.frame(wavelength = nm))}

# ------------------------------------------------------
# plot the individual color percentages (Figure 1)
# ------------------------------------------------------
plot1 <- function() {
	par(mfrow=c(2,1))	# two graphs per figure
	# red and green percentages
	plot(red ~ wavelength, data = df,
		col = "red", type = "o", pch = 19,
		xlim = c(400, 700),
		ylim = c(0, 100),
		xlab = "Wavelength in nm",
		ylab = "Percentage of Color",
		main = "Fig 1a: Red and Green Responses")
	points(green ~ wavelength, data = df,
		col = "green", type = "o", pch = 19,
		xlim = c(400, 700),
		ylim = c(-100, 100))

	# yellow and blue percentages
	plot(yellow ~ wavelength, data = df,
		col = "orange", type = "o", pch = 19,
		xlim = c(400, 700),
		ylim = c(0, 100),
		xlab = "Wavelength in nm",
		ylab = "Percentage of Color",
		main = "Fig 1b: Yellow and Blue Responses")
	points(blue ~ wavelength, data = df,
		col = "blue", type = "o", pch = 19,
		xlim = c(400, 700),
		ylim = c(-100, 100))
	par(mfrow=c(1,1))
}
	
# ------------------------------------------------------
# plot the opponent processes (Figure 2)
# ------------------------------------------------------
plot2 <- function() {
	par(mfrow=c(2,1))	# two graphs per figure
	# plot the red-green process
	df.red <- subset(df, red_green > 0)
	df.green <- subset(df, red_green <= 0)
	plot(red_green ~ wavelength, data = df.red,
		type = "p", col = "red", pch = 19,
		xlim = c(400, 700),
		ylim = c(-100, 100),
		xlab = "Wavelength in nm",
		ylab = "Red - Green Response",
		main = "Fig 2a: Red - Green Chromatic Mechanism")
	points(red_green ~ wavelength, data = df.green,
		type = "p", col = "green", pch = 19)
	abline(h = 0)

	# plot the best-fitting line from the polynomial
	x <- 400:700
	lines(x, predict(rg.poly, data.frame(wavelength = x)), lwd = 2)
	abline(v = pure.blue, col = "blue")
	abline(v = pure.yellow, col = "orange")

	# plot the yellow-blue opponent process
	df.yellow <- subset(df, yellow_blue > 0)
	df.blue <- subset(df, yellow_blue <= 0)
	plot(yellow_blue ~ wavelength, data = df.yellow,
		type = "p", col = "orange", pch = 19,
		xlim = c(400, 700),
		ylim = c(-100, 100),
		xlab = "Wavelength in nm",
		ylab = "Yellow - Blue Response",
		main = "Fig 2b: Yellow - Blue Chromatic Mechanism")
	points(yellow_blue ~ wavelength, data = df.blue,
		type = "p", col = "blue", pch = 19)
	abline(h = 0)

	# plot the best-fitting line from the polynomial
	x <- 400:700
	lines(x, predict(yb.poly, data.frame(wavelength = x)), lwd = 2)
	abline(v = pure.green, col = "green")
	par(mfrow=c(1,1))
}

# ------------------------------------------------------
# plot the opponent processes on one figure (Figure 3)
# ------------------------------------------------------
plot3 <- function() {
	par(mfrow=c(1,1))	# one graph per figure
	# plot the red-green process
	df.red <- subset(df, red_green > 0)
	df.green <- subset(df, red_green <= 0)
	plot(red_green ~ wavelength, data = df.red,
		type = "p", col = "red", pch = 19,
		xlim = c(400, 700),
		ylim = c(-100, 100),
		xlab = "Wavelength in nm",
		ylab = "Red - Green Response",
		main = "Fig 3: Chromatic Mechanisms")
	points(red_green ~ wavelength, data = df.green,
		type = "p", col = "green", pch = 19)
	abline(h = 0)

	# plot the best-fitting line from the polynomial
	x <- 400:700
	lines(x, predict(rg.poly, data.frame(wavelength = x)), lwd = 2)
	abline(v = pure.blue, col = "blue")
	abline(v = pure.yellow, col = "orange")

	# plot the yellow-blue opponent process
	df.yellow <- subset(df, yellow_blue > 0)
	df.blue <- subset(df, yellow_blue <= 0)
	points(yellow_blue ~ wavelength, data = df.yellow,
		type = "p", col = "orange", pch = 19)
	points(yellow_blue ~ wavelength, data = df.blue,
		type = "p", col = "blue", pch = 19)
	abline(h = 0)

	# plot the best-fitting line from the polynomial
	x <- 400:700
	lines(x, predict(yb.poly, data.frame(wavelength = x)), lwd = 2)
	abline(v = pure.green, col = "green")
	par(mfrow=c(1,1))
}

# ------------------------------------------------------
# plot the total percentages (Figure 4)
# ------------------------------------------------------
plot4 <- function() {
	par(mfrow=c(2,1))	# two graphs per figure
	plot(total ~ wavelength, data = df, pch = 19,
		ylim = c(40, 140),
		xlab = "Wavelength in nm",
		ylab = "Total Percentage",
		main = "Fig 4a: Sum of Red, Yellow, Green, and Blue Percentages")
	abline(h = 100)
	# compute a linear regression on total
	tot.lm <- lm(total ~ wavelength, data = df)
	lines(tot.lm$fitted ~ df$wavelength)

	# plot the residual from the total ~ wavelength regression
	plot(tot.lm$residual ~ df$wavelength, pch = 19,
		ylim = c(-50, 50),
		xlab = "Wavelength in nm",
		ylab = "Residuals",
		main = "Fig 4b: Residuals from Linear Regression")
	abline(h=0)
	par(mfrow=c(1,1))
}
